@@ -2,12 +2,12 @@ import collections
 import copy
 import difflib
 import logging
+import multiprocessing
 import pathlib
 from typing import (
     Collection, DefaultDict, Dict, Iterable, List, Optional, Tuple,
 )
 
-import yaml
 import yaml.scanner
 
 import square.cfgfile
@@ -23,6 +23,22 @@ logit = logging.getLogger("square")
 DotDict = square.dotdict.DotDict
 
 
+# Use the fast CSafeLoader/CSafeDumper from the LibYAML C library if they are
+# available on the host, otherwise fall back to the slow Python loader/dumper.
+# NOTE: this import is excluded from the coverage report since it is difficult
+#       to write a meaningful test around a library import that may or may not
+#       exist on the host. I deem this acceptable in this case because it is a
+#       widely used snipped devoid of logic.
+try:                                 # codecov-skip
+    from yaml import (  # type: ignore
+        CSafeDumper as Dumper, CSafeLoader as Loader,
+    )
+    logit.debug("Using LibYAML C library")
+except ImportError:                  # codecov-skip
+    from yaml import Dumper, Loader  # type: ignore
+    logit.debug("Using Python YAML library")
+
+
 def make_meta(manifest: dict) -> MetaManifest:
     """Compile `MetaManifest` information from `manifest` and return it.
 
@@ -177,6 +193,22 @@ def unpack_list(manifest_list: dict,
     return (manifests, False)
 
 
+def _parse_worker(fname: Filepath,
+                  yaml_str: str) -> Tuple[Collection[dict], bool]:
+    logit.debug(f"Parsing <{fname}>")
+
+    # Decode the YAML documents in the current file.
+    try:
+        manifests = list(yaml.load_all(yaml_str, Loader=Loader))
+        return manifests, False
+    except (yaml.parser.ParserError, yaml.scanner.ScannerError) as err:
+        logit.error(
+            f"Cannot YAML parse <{fname}>"
+            f" - {err.problem} - Line {err.problem_mark.line}"
+        )
+        return ([], True)
+
+
 def parse(
         file_yaml: Dict[Filepath, str],
         selectors: Selectors) -> Tuple[LocalManifestLists, bool]:
@@ -197,36 +229,36 @@ def parse(
     # The output dict will have a list of tuples.
     out: LocalManifestLists = {}
 
-    # Parse the YAML documents from every file.
-    for fname, yaml_str in file_yaml.items():
-        logit.debug(f"Parsing <{fname}>")
+    # Parse the YAML documents from every file. Use a process pool to speed up
+    # the process.
+    with multiprocessing.Pool() as pool:
+        # Compile the arguments for the worker processes that we will use to
+        # load the YAML files.
+        funargs = sorted(file_yaml.items())
+        fnames = sorted(list(file_yaml))
 
-        # Decode the YAML documents in the current file.
-        try:
-            manifests = list(yaml.safe_load_all(yaml_str))
-        except (yaml.parser.ParserError, yaml.scanner.ScannerError) as err:
-            logit.error(
-                f"Cannot YAML parse <{fname}>"
-                f" - {err.problem} - Line {err.problem_mark.line}"
-            )
-            return ({}, True)
+        # Parse the YAMLs in a process pool.
+        ret: Collection[Tuple[MetaManifest, dict]]
+        for fname, (manifests, err) in zip(fnames, pool.starmap(_parse_worker, funargs)):
+            if err:
+                return ({}, err)
 
-        # Remove all empty manifests. This typically happens when the YAML
-        # file ends with a "---" string.
-        manifests = [_ for _ in manifests if _ is not None]
+            # Remove all empty manifests. This typically happens when the YAML
+            # file ends with a "---" string.
+            manifests = [_ for _ in manifests if _ is not None]
 
-        # Retain only those manifests that satisfy the selectors.
-        manifests = [_ for _ in manifests if select(_, selectors)]
+            # Retain only those manifests that satisfy the selectors.
+            manifests = [_ for _ in manifests if select(_, selectors)]
 
-        # Convert List[manifest] into List[(MetaManifest, manifest)].
-        # Abort if `make_meta` throws a KeyError which happens if `file_yaml`
-        # does not actually contain a Kubernetes manifest but some other
-        # (valid) YAML.
-        try:
-            out[fname] = [(make_meta(_), _) for _ in manifests]
-        except KeyError:
-            logit.error(f"{file_yaml} does not look like a K8s manifest file.")
-            return {}, True
+            # Convert List[manifest] into List[(MetaManifest, manifest)].
+            # Abort if `make_meta` throws a KeyError which happens if `file_yaml`
+            # does not actually contain a Kubernetes manifest but some other
+            # (valid) YAML.
+            try:
+                out[fname] = [(make_meta(_), _) for _ in manifests]
+            except KeyError:
+                logit.error(f"{file_yaml} does not look like a K8s manifest file.")
+                return {}, True
 
     # Drop all files without manifests.
     out = {k: v for k, v in out.items() if len(v) > 0}
@@ -443,8 +475,8 @@ def diff(config: Config,
     # Precaution: undo the DotDicts to ensure the YAML parse will accept them.
     srv = square.dotdict.undo(server)
     loc = square.dotdict.undo(local)
-    srv_lines = yaml.dump(srv, default_flow_style=False).splitlines()
-    loc_lines = yaml.dump(loc, default_flow_style=False).splitlines()
+    srv_lines = yaml.dump(srv, default_flow_style=False, Dumper=Dumper).splitlines()
+    loc_lines = yaml.dump(loc, default_flow_style=False, Dumper=Dumper).splitlines()
 
     # Compute and return the lines of the diff.
     diff_lines = difflib.unified_diff(srv_lines, loc_lines, lineterm='')
@@ -904,7 +936,8 @@ def save(folder: Filepath, manifests: LocalManifestLists,
     out_final: Dict[Filepath, str] = {}
     try:
         for fname, v in out_clean.items():
-            out_final[fname] = yaml.safe_dump_all(v, default_flow_style=False)
+            out_final[fname] = yaml.dump_all(v, default_flow_style=False,
+                                             Dumper=Dumper)
     except yaml.YAMLError as e:
         logit.error(
             f"YAML error. Cannot create <{fname}>: {e.args[0]} <{str(e.args[1])}>"
